from EEUci import EEUci
import simplejson as json
import sys
import getopt

onboard_file="/opt/novatec/data/slave/json_data/raw/onboard.json"
uci_config="machines"
uci_gauge_config="gaugerules"
uci_baselines="baselines"
uci_calibration_config="novatec_sensors"

sensor_packages = {
    "AT":"ST",
    "BP": "ST",
    "IR": "ST",
    "HY": "ST",
    "GY": "ST",
    "AC": "ST",
    "MG": "ST"
}

assembly_map = {                                                     
    "rail": "railcar",                                                   
    "pump": ["vacuum-pump","machinesense","vacuumsense"],                                               
    "component_analyzer": ["componentanalyzer","componentanalyzer"],
    "pressuresense": "pressuresense",
    "motorsense": ["motorsense2port","motorsense4port","motorsense"],
    "heatersense": ["heatersense2port","heatersense4port","heatersense"],
    "energysense": ["energysense2port","energysense4port","energysense"],
    "vfdsense": ["vfdsensev2","vfdinput","vfdoutput"],      
    "cassette_motor": "cassettemotor",                                   
    "cassette_chain": "cassettechain",                                   
    "regen_blower": "regenblower",            
    "process_blower": "processblower",
    "regen_heater": "dryerregenheater",
    "cda_pf": "cdaprocessheater",     
    "cda_ps": "boosterblower",
    "plc": "plc"                      
}


def find_subtype(assembly_name):
    for key,value in assembly_map.items():
        if assembly_name.lower() in value:
            return key

def get_product_type(subtype):
    if 'rail' in subtype:
        return subtype
    if 'plc' in subtype:
        return subtype
    if 'pump' in subtype or any([x in subtype for x in ["cassette","blower","process","component","cda_ps"]]):
        return 'pump'
    if any([x in subtype for x in ["heater","motor","energy","vfd","cda_pf"]]):
        return 'power'
    

def tag_ST(sensor_name):
    name=sensor_packages.get(sensor_name[:2].upper(),None)
    return name.lower() if name else sensor_name

def get_sensor_package_type(sid):
    s=sid.upper()
    if s in sensor_packages:
        return sensor_packages[s]
    return s

def find_sensor(machines,sid):
    for mid,m in machines.items():
        for sensor in m['sensors']:
            if sid in sensor['id']:
                return sensor
    return None

def get_models(machine):
    models=machine["models"];model={}
    model["pump_model"]=models.get("name","VPDB-10_V3").lower()
    model["blower_type"]=models.get("blowerType","PD (Straight 2-Lobe)")
    model["vendor_name"]=models.get("vendorName","Gardner Denver")
    model["blower_model"]=models.get("blowerModel","4M-DSL")
    model["blower_serial_number"]=models.get("blowerSerialNumber","a100")
    return model

def get_kfactor(gauge_data,mid):
    for gd in gauge_data:
        if gd['id']==mid and 'k_factor' in gd:
            return {'k_factor':gd['k_factor']}
    return {}

def update_sensor(sensor_id,key,value):
    s = find_sensor(uciMachines,sensor_id)
    s[key]=value

def get_sensors(sensorAssignments,mid,machine):
    machine_sensors=[]
    for sa in sensorAssignments:
        sensor={}
        sensorId=sa['sensorId']
        s=sensorId.split("_")
        package_type = get_sensor_package_type(s[1])
        sensorMac=f"{s[0][0:2]}:{s[0][2:4]}:{s[0][4:6]}:{s[0][6:8]}:{s[0][8:10]}:{s[0][10:12]}".upper()
        sensor['mac']=sensorMac
        sensor['index']=s[2]
        if machine['type'] in ["power","plc"]:
            sensor['port']=s[2]
        sensor['id']=f"{s[0]}_{package_type}_{s[2]}"
        sensor['type'] = package_type
        sensor['machine'] = mid
        sensor['name'] = tag_ST(sa['collectors'].get('name','Unknown'))
        set_vfd(machine['subtype'],sensor)
        set_cda(machine['subtype'],sensor,s[0])
        if sensor not in machine_sensors:
            machine_sensors.append(sensor)
    return machine_sensors

def check_vfd(subtype):
    if subtype=="vfdsense":
        return True
    return False

def set_cda(subtype,sensor,mac):
    if subtype=="plc":
        if int(sensor['index'])>=2:
            sensor['index']=str(int(sensor['index'])-1)
            sensor['type'] = "CDA"
            sensor['id']=f"{mac}_{sensor['type']}_{sensor['index']}"
            sensor['name'] = f"{sensor['type']}_{sensor['index']}"
            sensor['port']=sensor['index']

def set_vfd(subtype,sensor):
    if check_vfd(subtype):
        if int(sensor['port'])==1:
            sensor["io_type"]="input"  
        elif int(sensor['port'])==2:
            sensor["io_type"]="output"

def get_machines(machines,gauge_data):
    uciMachines={}
    for machine in machines:
        m={}
        mid = machine['id']
        m['id'] = mid
        m['name'] = machine.get('name',"Unknown")
        m['subtype']=find_subtype(machine["models"]["assembly"].get("name","vacuum-pump"))
        m['type']=get_product_type(m['subtype'])
        m[f'subtype_{m["type"]}']=m['subtype']
        m['sensors']=get_sensors(machine['sensorAssignments'],mid,m)
        m['manufacturer_type']=machine.get("category","oem").lower()
        if 'pump_number' in machine:
            m['pump_number'] = machine.get('pumpNumber',"1")
        if "part_number" in machine:
            m["part_number"]=machine.get('part_number',"NA")
        if 'description' in machine:
            m['description'] = machine.get('description',"pumpsense device")
        m.update(get_kfactor(gauge_data,mid))
        if 'pump' in m['type']:
            m.update(get_models(machine))
        uciMachines[mid] = m
    print("[+] Formed UCI Machines Struct: ",json.dumps(uciMachines,indent=4))
    return uciMachines

def get_qc_params(qc_params):
    for qc in qc_params:
        if 'calibrationParam' in qc: 
            cp = qc['calibrationParam']
            if 'calibration_parameters' in cp:
                for key,value in cp['calibration_parameters'].items():
                    if "pf" in qc['id'].lower():
                        update_sensor(qc['id'],key,list(value.values())[0]);continue
                    update_sensor(qc['id'],f"calib_{key}",value)

def get_sensor_boards(sensor_boards):
    for sb in sensor_boards:
        for sensor in sb["sensors"]:
            if 'cttype' in sensor:
                update_sensor(sensor['id'],"cttype",sensor["cttype"])
                update_sensor(sensor['id'],"loadtype",sensor["loadtype"])
                update_sensor(sensor['id'],"isostat",sensor["isostat"])
                update_sensor(sensor['id'],"phasetype",sensor.get("phasetype","Three"))

def get_gauges(gauges):
    uciGauges=[]
    for gauge_details in gauges:
        g={}
        mid=gauge_details['id']
        g['id']=mid
        g["rules"]=[]
        if 'kFactor' in gauge_details:
            g['k_factor']=gauge_details['kFactor']
        #print(g)
        for gi in gauge_details["gaugeUserInputs"]:
            gauge={}
            gauge["pm_function"]=gi["pmFunction"]["name"]
            gauge["subassembly"]=gi["subAssemblyInstance"]["name"]
            gauge["name"]=gi["userRequiredInput"]["name"]
            gauge["value"]=gi["value"]
            if gauge not in g["rules"]:
                g["rules"].append(gauge)
        if "baselines" in gauge_details:
            g["baselines"]=[]
            for gi in gauge_details["baselines"]:
                gauge={}
                gauge["pm_function"]=gi.get("pm_function","unknown")
                gauge["subassembly"]=gi.get("subassembly","unknown")
                gauge["name"]=gi.get("name","unknown")
                gauge["value"]=gi.get("value","unknown")
                if gauge not in g["baselines"]:
                    g["baselines"].append(gauge)
        uciGauges.append(g)
    print("[+] Formed Gauge Rule Struct: ",json.dumps(uciGauges,indent=4))
    return uciGauges


def gen_uci_machines(c,uciMachines):
    u = EEUci()
    with open("/etc/config/"+c,"w") as f:
        pass
    for key,machine in uciMachines.items():
        n=key.replace("-","")
        print("[+] Writing uci_config for rule: ",n)
        u.set(c,n,"machine")
        for k,value in machine.items():
            if k == "sensors":
                continue
            print("[+] Writing uci_config for option: ",n)
            u.set(c,n,k,value)
        for sensor in machine['sensors']:
            sensor_key=f"{sensor['machine'].replace('-','')}_{sensor['id']}"
            print("[+] Writing uci_config for rule: ",sensor_key)
            u.set(c,sensor_key,"sensor")
            for sk,sval in sensor.items():
                print("[+] Writing uci_config for option: ",sensor_key)
                u.set(c,sensor_key,sk,sval)
    u.commit(c)
    print("[*] Commited Machine Rules.")

def gen_uci_gauge_rule(c,data):
    u = EEUci()
    with open("/etc/config/"+c,"w") as f:
        pass
    for d in data:
        if 'rules' not in d:
            return
        for rule in d['rules']:
            rule_key=f"{d['id'].replace('-','')}_{rule['pm_function'].replace(' ','_').lower()}_{rule['name']}"
            #rule_key=u.get(c)
            print("[+] Writing uci_gauge_config for rule: ",rule_key)
            u.set(c,rule_key,"rule")
            u.set(c,rule_key,'machine_id', d['id'])
            u.set(c,rule_key,'subassembly', rule['subassembly'])
            u.set(c,rule_key,'name', rule['name'])
            u.set(c,rule_key,'pm_function', rule['pm_function'])
            u.set(c,rule_key,'value', rule['value'])
    u.commit(c)
    print("[*] Commited Gauge Rules.")

def gen_uci_baselines(c,data):
    u = EEUci()
    with open("/etc/config/"+c,"w") as f:
        pass
    for d in data:
        if 'baselines' not in d:
            return
        for baseline in d['baselines']:
            baseline_key=f"{d['id'].replace('-','')}_{baseline['pm_function'].replace(' ','_').lower()}_{baseline['name']}"
            print("[+] Writing baselines for gauge: ",baseline_key)
            u.set(c,baseline_key,"baseline")
            u.set(c,baseline_key,'machine_id', d['id'])
            u.set(c,baseline_key,'subassembly', baseline['subassembly'])
            u.set(c,baseline_key,'name', baseline['name'])
            u.set(c,baseline_key,'pm_function', baseline['pm_function'])
            u.set(c,baseline_key,'value', baseline['value'])
    u.commit(c)
    print("[*] Commited Gauge Baselines.")

def gen_uci_calibration_config(c,uciMachines):
    u = EEUci()
    with open("/etc/config/"+c,"w") as f:
        pass
    for key,machine in uciMachines.items():
        for sensor in machine['sensors']:
            sensor_key=f"{sensor['id']}"
            for sk,sval in sensor.items():
                if "calib" in sk or "voltage" in sk or "current" in sk:
                    print("[+] Writing uci_calibration_config for option: ",sensor_key)
                    u.set(c,sensor_key,"sensor")
                    u.set(c,sensor_key,sk,sval)
    u.commit(c)
    print("[*] Commited Calibration Config Rules.")

def is_baseline(gauge_details):
    for gd in gauge_details:
        if "baselines" in gd:
            return True
    return False

def usage():
    print(f"usage: {sys.argv[0]} [-?] [-p product-type] [-s subproduct-type]")
    sys.exit()

try:
    opts, args = getopt.getopt(sys.argv[1:],"?o:")
except getopt.GetoptError:
    usage()

for opt,arg in opts:
    if opt == "-o":
        onboard_file = arg
    if opt == "-?":
        usage()


with open(onboard_file,"r") as f:
	onboard_json=json.load(f)


#print(json.dumps(onboard_json,indent=4))

machines=onboard_json['machines']
qc_params=onboard_json['qc_params']


gauge_data=get_gauges(onboard_json['gauges'])
uciMachines=get_machines(machines,gauge_data)


get_qc_params(qc_params)
if "sensor_boards" in onboard_json:
    sensor_boards=onboard_json["sensor_boards"]
    get_sensor_boards(sensor_boards)
gen_uci_machines(uci_config,uciMachines)
gen_uci_gauge_rule(uci_gauge_config,gauge_data)
if is_baseline(gauge_data):
    gen_uci_baselines(uci_baselines,gauge_data)
gen_uci_calibration_config(uci_calibration_config,uciMachines)

