import getmac
import sys
import simplejson as json

def capitalise(s):
    return s[0].upper()+s[1:]

def firstLower(s):
    return s[0].lower()+s[1:]

name_map = {
    "machinesense": "machineSense",
    "vacuumsense": "vacuumSense",
    "component_analyzer": "componentAnalyzer",
    "pressuresense": "pressureSense",
    "motorsense": "MotorSense",
    "heatersense": "HeaterSense",
    "energysense": "EnergySense",
    "motorsense2port": "motorSense2Port",
    "heatersense2port": "heaterSense2Port",
    "energysense2port": "energySense2Port",
    "motorsense4port": "motorSense4Port",
    "heatersense4port": "heaterSense4Port",
    "energysense4port": "energySense4Port",
    "vfdsense": "VfdSenseV2",
    "vfdinput": "vfdInput",
    "vfdoutput": "vfdOutput"
}

assembly_info = {
        "cassette_motor": {
        "name": "CassetteMotor",
        "display_name": "Desiccant Wheel Motor",
        "assemblyName": "DSMCassette",
        "collectors":["AC","IR","HY","AT","BP","MG","GY"] 
        },
        "cassette_chain": {
        "name": "CassetteChain",
        "display_name": "Desiccant Wheel Alignment",
        "assemblyName": "DSCCassette",
        "collectors": ["AC","IR","HY","AT","BP","MG","GY"]
        },
        "regen_blower": {
        "name": "RegenBlower",
        "display_name": "Regen Blower Health",
        "assemblyName": "DSRBlowers",
        "collectors": ["AC","IR","HY","AT","BP","MG","GY"],
        },
        "process_blower": {
        "name": "ProcessBlower",
        "display_name": "Process Blower Health",
        "assemblyName": "DSPBlowers",
        "collectors": ["AC","IR","HY","AT","BP","MG","GY"],
        },
        "regen_heater": {
        "name": "DryerRegenHeater",
        "display_name": "Regen Heater Health (Stage 1)",
        "assemblyName": "DryerRegenHeater_1Port",
        "collectors": ["PF"],
        },
        "cda_pf": {
        "name": "CDAProcessHeater",
        "display_name": "Regen Heater Health (Stage 1)",
        "assemblyName": "CDAHeater",
        "collectors": ["PF"],
        },
        "cda_ps": {
        "name": "BoosterBlower",
        "display_name": "Regen Heater Health (Stage 1)",
        "assemblyName": "CDABlower",
        "collectors": ["AC","IR","HY","AT","BP","MG"],
        },
        "component_analyzer": {
        "name": "ComponentAnalyzer",
        "display_name": "Vibration Analyzer",
        "assemblyName": "ComponentAnalyzer",
        "collectors": ["AC","IR","HY","AT","GY","BP","MG"],
        }
    }

def get_assembly_info_map(assembly_type):
    return assembly_info.get(assembly_type,{"name": None,"display_name":None,"assemblyName":None})

def map_assembly_name(name):
    n = name_map.get(name.lower())
    if n:
        return n
    return name

def get_map_query(name,port):
    return f"{name}{str(port)}port" if port>1 else name

class SensorPackage:
    def __init__(self,cfg,index=1):
        self.cfg = cfg
        self.mac = cfg['mac']
        self.index = index
        if 'index' in self.cfg:
            self.index=self.cfg['index']
        if 'port' in self.cfg:
            self.index=self.cfg['port']
        self.uuid=self.mac.replace(":","").lower()
        self.type = self.cfg['type'].upper()
        self.sensorId = f'{self.uuid}_{self.type}_{self.index}'
        self.sensorName = self.cfg.get('name','Unknown')
       
    def gen_collector(self,type,index,description=None):
        name= f"{type}{index}"
        if not description:
            description=name.lower()
        result = {
                "name":name.lower(),
                "lowercase_name": name.lower(),
                "display_name": name.upper(),
                "type": type,
                "description": description,
              }
        return result

    def getMac(self):
        return self.mac

    def gen_collectors(self):
        pass

    def gen_subassemblyInstance(self):
        pass 
 
class VACSensorPackage (SensorPackage):
    def __init__(self,mac=None,subtype="pump",machinetype="pump",**kwargs):
        super().__init__(mac)
        subassemblyInstanceName="vacuumsense"
        self.subassemblyName=map_assembly_name(subassemblyInstanceName) if machinetype!="pump" else subassemblyInstanceName
        self.name=self.subassemblyName+str(self.index)
        self.display_name="Vacuum Analyzer"
        self.collectors=["VAC"]


    def gen_collectors(self):
        collectors=[]
        collectors.append(self.gen_collector("VAC",self.index))
        return collectors

    def gen_subassemblyInstance(self):
        result = {
          "name": self.name,
          "lowercase_name": self.name.lower(),
          "display_name": self.display_name,
          "subAssembly": {
              "name": "vacuumSense",
              "lowercase_name": "vacuumsense",
              "display_name": "VacuumAnalyzer",
              "description": "Vacuum Diagnosis and Vacuum Analytics",
              "version": 0.02,
              "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors()
        return result

class PSSensorPackage (VACSensorPackage):
    def __init__(self,mac=None,subtype="pump",machinetype="rail",**kwargs):
        super().__init__(mac)
        subassemblyInstanceName="pressuresense"
        self.subassemblyName=map_assembly_name(subassemblyInstanceName) if machinetype!="pump" else subassemblyInstanceName
        self.name=self.subassemblyName+str(self.index)
        self.display_name=self.subassemblyName+str(self.index)
        self.collectors=["PS"]
    
    def gen_collectors(self):
        collectors=[]
        collectors.append(self.gen_collector("PS",self.index))
        return collectors

    def gen_subassemblyInstance(self):
        result = {
          "name": self.name,
          "lowercase_name": self.name.lower(),
          "display_name": self.display_name,
          "subAssembly": {
              "name": "pressureSense",
              "lowercase_name": "pressuresense",
              "display_name": "pressureSense",
              "description": "Pressure Diagnosis and Pressure Analytics",
              "version": 0.02,
              "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors()
        return result

class STSensorPackage (SensorPackage):
    def __init__(self,mac=None,subtype="pump",machinetype="pump",index=1,**kwargs):
        super().__init__(mac,index=index)
        subassemblyInstanceName="machinesense"
        self.subassemblyName=map_assembly_name(subassemblyInstanceName) if machinetype!="pump" else subassemblyInstanceName
        self.name=self.subassemblyName+str(self.index)
        self.display_name="Vibration Analyzer"
        self.collectors=["AC","IR","HY","AT","BP","MG","GY"]

        pass

    def gen_collectors(self):
        collectors=[]
        collectors.append(self.gen_collector("AC",1))
        collectors.append(self.gen_collector("IR",1))
        collectors.append(self.gen_collector("HY",1))
        collectors.append(self.gen_collector("AT",1))
        collectors.append(self.gen_collector("BP",1))
        collectors.append(self.gen_collector("MG",1))
        collectors.append(self.gen_collector("GY",1))
        return collectors

    def gen_subassemblyInstance(self):
        result = {
            "name": self.name,
            "lowercase_name": self.name.lower(),
            "display_name": self.display_name,
            "subAssembly": {
            "name": "machineSense",
            "lowercase_name": "machinesense",
            "display_name": "Vibration Analyzer",
            "description": "Tracking abnormal vibration, misalignment, machine temp, ambient temp, humidity",
            "version": 0.02,
            "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors()
        return result


class DryerSensorPackage (STSensorPackage):
    def __init__(self,mac=None,subtype="cassette_motor",machinetype="pump"):
        super().__init__(mac)
        self.assembly_info=get_assembly_info_map(subtype)
        self.name=self.assembly_info["name"]
        self.display_name=self.assembly_info["display_name"]
        self.subassemblyName=self.name+str(self.index)
        self.collectors=self.assembly_info["collectors"]

    def update_names(self,name,display_name,collectors):
        self.name=name
        self.subassemblyName = name + str(self.index)
        self.display_name= display_name
        self.collectors=collectors
    
    def gen_collectors(self,typelist):
        collectors=[]
        for type in typelist:
            collectors.append(self.gen_collector(type,1))
        return collectors
        
    def gen_subassemblyInstance(self):
        result = {
            "name": self.subassemblyName,
            "lowercase_name": self.subassemblyName.lower(),
            "display_name": self.display_name,
            "subAssembly": {
            "name": self.name,
            "lowercase_name": self.name.lower(),
            "display_name": self.name,
            "description": f"subAssembly for {self.name}",
            "version": 1,
            "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors(self.collectors)
        return result

class ComponentAnalyzerSensorPackage(STSensorPackage):
    def __init__(self,mac=None,subtype="component_analyzer",machinetype="pump",index=1):
        super().__init__(mac,index=index)
        self.assembly_info=get_assembly_info_map(subtype)
        self.name=self.assembly_info["name"]
        self.display_name=self.assembly_info["display_name"]
        self.subassemblyName=self.name+str(self.index)
        self.collectors=self.assembly_info["collectors"]

    def update_names(self,name,display_name,collectors):
        self.name=name
        self.subassemblyName = name + str(self.index)
        self.display_name= display_name
        self.collectors=collectors
    
    def gen_collectors(self,typelist):
        collectors=[]
        for type in typelist:
            collectors.append(self.gen_collector(type,1))
        return collectors
        
    def gen_subassemblyInstance(self):
        result = {
            "name": firstLower(self.subassemblyName),
            "lowercase_name": self.subassemblyName.lower(),
            "display_name": self.display_name,
            "subAssembly": {
            "name": firstLower(self.name),
            "lowercase_name": self.name.lower(),
            "display_name": self.name,
            "description": "Tracking abnormal vibration, misalignment, machine temp, ambient temp, humidity",
            "version": 1,
            "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors(self.collectors)
        return result
    
#def gen_vacuum():
#    return VACSensorPackage().gen_subassemblyInstance()


#def gen_st():
#     return STSensorPackage().gen_subassemblyInstance()

class PFSensorPackage (SensorPackage):
    def __init__(self,sensor=None,subtype="motorSense",machinetype="power",**kwargs):
        super().__init__(sensor)
        self.subassemblyName = map_assembly_name(subtype)
        self.name=map_assembly_name(subtype) + self.index
        self.display_name="Electral Panel1"
        self.collectors=["PF"]

    def setSubassemblyName(self,name):
        self.subassemblyName = map_assembly_name(name)  
        self.update_names()

    def update_names(self):
        self.name=self.subassemblyName + self.index
        self.name = firstLower(self.name)     # This is just more kludge
        self.display_name="Electral Panel1"
        self.collectors=["PF"]

    def gen_collectors(self):
        collectors=[]
        collectors.append(self.gen_collector("PF",1))
        return collectors

    def gen_subassemblyInstance(self):
        result = {
          "name": self.name,
          "lowercase_name": self.name.lower(),
          "display_name": self.display_name,
          "subAssembly": {
              "name": firstLower(self.subassemblyName),
              "lowercase_name": self.subassemblyName.lower(),
              "display_name": "Electrical Panel",
              "description": "Predictive maintenance of 3 Phase AC motor + Energy /Cost/Productivity analytics",
              "version": 0.02,
              "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors()
        return result

class VFDISensorPackage (PFSensorPackage):
    def __init__(self,sensor=None,subtype="vfdsense",machinetype="power",sensortype="vfdinput",**kwargs):
        sensor['type']='PF'
        super().__init__(sensor)
        self.subassemblyName = map_assembly_name(sensortype)
        self.name=map_assembly_name(sensortype) + self.index
        self.display_name="Electral Panel1"
        self.collectors=["PF"]

    def setSubassemblyName(self,name): 
        self.update_names()
    
    def gen_subassemblyInstance(self):
        result = {
          "name": self.name,
          "lowercase_name": self.name.lower(),
          "display_name": self.display_name,
          "subAssembly": {
              "name": self.subassemblyName,
              "lowercase_name": self.subassemblyName.lower(),
              "display_name": self.subassemblyName,
              "description": "Electrical Panel-Motor-VFD",
              "version": 0.02,
              "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors()
        return result

class VFDOSensorPackage (VFDISensorPackage):
    def __init__(self,sensor=None,subtype="vfdsense",machinetype="power",sensortype="vfdoutput",**kwargs):
        super().__init__(sensor)
        self.subassemblyName = map_assembly_name(sensortype)
    
    def gen_subassemblyInstance(self):
        self.name=self.subassemblyName + str(int(self.index)-1)
        result = {
          "name": self.name,
          "lowercase_name": self.name.lower(),
          "display_name": self.display_name,
          "subAssembly": {
              "name": self.subassemblyName,
              "lowercase_name": self.subassemblyName.lower(),
              "display_name": self.subassemblyName,
              "description": "Electrical Panel-Motor-VFD",
              "version": 0.02,
              "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors()
        return result

class PLCSensorPackage (SensorPackage):
    def __init__(self,sensor=None,subtype="plc",machinetype="plc",**kwargs):
        super().__init__(sensor)
        self.subassemblyName = subtype
        self.name="DryerPLC"+str(self.index)
        self.display_name=self.name
        self.collectors=["PLC"]
    
    def set_subassembly_details(self,name):
        return name+str(self.index)

    def setSubassemblyName(self,name):
        self.subassemblyName = "DryerPLC"
        self.name=self.set_subassembly_details(self.subassemblyName)
        self.display_name=self.set_subassembly_details(self.subassemblyName)
        self.update_names()

    def update_names(self):
        self.name=self.name
        self.display_name=self.display_name
        self.collectors=["PLC"]

    def gen_collectors(self):
        collectors=[]
        collectors.append(self.gen_collector("PLC",1))
        return collectors
    
    def gen_subassemblyInstance(self):
        result = {
          "name": self.name,
          "lowercase_name": self.name.lower(),
          "display_name": self.display_name,
          "subAssembly": {
              "name": self.subassemblyName,
              "lowercase_name": self.subassemblyName.lower(),
              "display_name": self.subassemblyName,
              "description": "subAssembly for PLC",
              "version": 1,
              "collectors": []
          }
        }
        result["subAssembly"]["collectors"] = self.gen_collectors()
        return result

class CDASensorPackage (PLCSensorPackage):
    def __init__(self,sensor=None,subtype="plc",machinetype="plc",**kwargs):
        sensor['type']='PLC'
        super().__init__(sensor)
        self.sensorId = f'{self.uuid}_{self.type}_{int(self.index)+1}'                        # Kludge to handle kludge logic, will remove it later.
        self.subassemblyName = subtype
        self.name="CdaPLC"+str(self.index).zfill(2)
        self.display_name=self.name
        self.collectors=["PLC"]

    def set_subassembly_details(self,name):
        return name+str(self.index).zfill(2)

    def setSubassemblyName(self,name):
        self.subassemblyName = "CdaPLC"
        self.name=self.set_subassembly_details(self.subassemblyName)
        self.display_name=self.set_subassembly_details(self.subassemblyName)
        self.update_names()

class Machine:
    def __init__(self,machine_id=None):
        if not machine_id:
            machine_id="machine-id-test"
        self.machine_id = machine_id

class PumpMachine(Machine):
    def __init__(self,slist=[],**kwargs):
        super().__init__(**kwargs)
        self.sensors=slist

    def gen_subassemblyInstances(self):
        instances=[]
        for sensor in self.sensors:
            instances.append(sensor.gen_subassemblyInstance())
        #instances.append(self.StSensor.gen_subassemblyInstance())
        #instances.append(self.VacSensor.gen_subassemblyInstance())
        return instances


    def gen_models(self):
        models ={
            "assembly": {
                "name": "vacuum-pump",
                "subAssemblyInstances":[],
            },
            "assemblyId": "assembly id stubbed",
            "name": "assembly name stubbed",
            "companyId": "StubbedModelId"
        }
        #models['assembly']['subAssemblyInstances'] = self.gen_subassemblyInstances()
        return models

    def gen_sensor_assignments(self,sensor,subassemblyInstance,machine_id):
        #print(json.dumps(subassemblyInstance,indent=4))
        #uuid=mac.replace(":","").lower()
        uuid=sensor.mac.replace(":","").lower()
        collectors = subassemblyInstance['subAssembly']['collectors']
        del subassemblyInstance['subAssembly']['collectors']
        l=[]
        for collector in collectors:
            sa={
                "machineId": machine_id,
                "sensorId":  f'{uuid}_{collector["type"]}_1',
                "collectors": {},
                "subAssemblyInstance": subassemblyInstance
            }
            #print("Collectors: ",collectors)
            collector=collector
            #collector["subAssembly"]=subassemblyInstance['subAssembly']
            sa['collectors']=collector
            l.append(sa)
        return l


    def gen_SensorAssignment(self,SensorPackage,machine_id):
        subassemblyInstance=SensorPackage.gen_subassemblyInstance()
        return self.gen_sensor_assignments(SensorPackage,subassemblyInstance,machine_id)

    def gen_sensorAssignments(self,machine_id):
        global vac
        assignments=[]
        for sensor in self.sensors:
            assignments = assignments + self.gen_SensorAssignment(sensor,machine_id)
        #assignments = assignments + self.gen_SensorAssignment(self.VacSensor,machine_id)
        #assignments = assignments + self.gen_SensorAssignment(self.StSensor,machine_id)
        return assignments


    def gen_zone(self):
        return {                                                                            
            "name": "Stubbed",
            "id": "id-stub",
            "location":  {                                                                   
                "name": "PumpSenseOnPremise",                                                
                "lowercase_name": "pumpsenseonpremise",                                      
                "city": "Kolkata",                                                           
                "state": "West Bengal",                                                      
                "country": "India",                                                          
                "display_name": "PumpSenseOnPremise",                                        
                "id": "ZoneIdStub",                                                          
                "companyId": "ZoneCompanyIdStub",                                            
                "company": {                                                                 
                    "name": "ProphecyDemo",                                                  
                    "lowercase_name": "prophecydemo",                                        
                    "display_name": "Prophecy Demo",                                         
                    "purpose": "civil",                                                      
                    "id": "company_id_stub"                                                  
                }                                                                            
            }                                                                                
        }

    def gen_machine(self,machine_id=None,name=None,category="oem",pumpNumber=None,pump_model=None,blower_type=None,vendor_name=None,blower_model=None,blower_serial_number=None,part_number=None,description="default"):
        if not name:
            name="StandAloneTest"
        if not machine_id:
            machine_id="machine-id-test"
        machine={ 
            "id": machine_id,
            "name": name,
            "lowercase_name": name.lower(),
            "display_name": name,
            "description": description,
            "category": category,
            "mode": "24x7",
            "group": "24x7",
	        "zoneId": "SubbedZoneId",
            "zone": self.gen_zone()
        }
        if part_number:
            machine['part_number'] = part_number
        if pumpNumber:
            machine['pumpNumber'] = pumpNumber
        machine['models']=self.gen_models()
        if pump_model:
            machine['models']['name']=pump_model
        if blower_type:
            machine['models']['blowerType']=blower_type
        if vendor_name:
            machine['models']['vendorName']=vendor_name
        if blower_model:
            machine['models']['blowerModel']=blower_model
        if blower_serial_number:
            machine['models']['blowerSerialNumber']=blower_serial_number
        machine['sensorAssignments']=self.gen_sensorAssignments(machine_id)
        return machine

class RailCarMachine(PumpMachine):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
    
    def gen_models(self):
        models ={
            "assembly": {
                "name": "RailCar",
                "subAssemblyInstances":[],
            },
            "assemblyId": "assembly id stubbed",
            "name": "assembly name stubbed",
            "companyId": "StubbedModelId"
        }
        #models['assembly']['subAssemblyInstances'] = self.gen_subassemblyInstances()
        return models
    
    def gen_zone(self):
        return {                                                                            
            "name": "Stubbed",
            "id": "id-stub",
            "location":  {                                                                   
                "name": "RailCarOnPremise",                                                
                "lowercase_name": "railcaronpremise",                                      
                "city": "Kolkata",                                                           
                "state": "West Bengal",                                                      
                "country": "India",                                                          
                "display_name": "RailCarOnPremise",                                        
                "id": "ZoneIdStub",                                                          
                "companyId": "ZoneCompanyIdStub",                                            
                "company": {                                                                 
                    "name": "ProphecyDemo",                                                  
                    "lowercase_name": "prophecydemo",                                        
                    "display_name": "Prophecy Demo",                                         
                    "purpose": "civil",                                                      
                    "id": "company_id_stub"                                                  
                }                                                                            
            }                                                                                
        }

class PowerMachine(PumpMachine):
    def __init__(self,*args,subtype="motorSense",max_port=1,**kwargs):
        super().__init__(*args,**kwargs)
        self.subassemblyName=subtype
        self.max_port=self.get_max_port(max_port)
        for sensor in self.sensors:
            sensor.setSubassemblyName(self.subassemblyName)
    
    def get_max_port(self,max_port):                                                                                       
        return 1 if self.subassemblyName=="vfdsense" else max_port

    def gen_models(self):
        models ={
            "assembly": {
                "name": map_assembly_name(get_map_query(self.subassemblyName,self.max_port)),
                "subAssemblyInstances":[],
            },
            "assemblyId": "assembly id stubbed",
            "name": "assembly name stubbed",
            "companyId": "StubbedModelId"
        }
        models['assembly']['subAssemblyInstances'] = self.gen_subassemblyInstances()
        return models

    def gen_sensor_assignments(self,sensor,subassemblyInstance,machine_id):
        #print(json.dumps(subassemblyInstance,indent=4))
        #uuid=mac.replace(":","").lower()
        collectors = subassemblyInstance['subAssembly']['collectors']
        del subassemblyInstance['subAssembly']['collectors']
        l=[]
        for collector in collectors:
            sa={
                "machineId": machine_id,
                "sensorId": sensor.sensorId,
                "collectors": {},
                "subAssemblyInstance": subassemblyInstance
            }
            #print("Collectors: ",collectors)
            collector=collector
            #collector["subAssembly"]=subassemblyInstance['subAssembly']
            sa['collectors']=collector
            l.append(sa)
        return l

class PLCMachine(PowerMachine):
    def __init__(self,slist=[],**kwargs):
        super().__init__(slist,**kwargs)
        self.sensors=slist
        for sensor in self.sensors:
            sensor.setSubassemblyName(sensor.sensorName)
    
    def gen_models(self):
        models ={
            "assembly": {
                "name": "PLC",
                "subAssemblyInstances":[],
            },
            "assemblyId": "assembly id stubbed",
            "name": "assembly name stubbed",
            "companyId": "StubbedModelId"
        }
        models['assembly']['subAssemblyInstances'] = self.gen_subassemblyInstances()
        return models

class DryerMachine(PumpMachine):
    def __init__(self,*args,subtype="cassette_motor",**kwargs):
        super().__init__(*args,**kwargs)
        self.assembly_info=get_assembly_info_map(subtype)
        for sensor in self.sensors:
            sensor.update_names(self.assembly_info["name"],self.assembly_info["display_name"],self.assembly_info["collectors"])
    
    def gen_models(self):
        models ={
            "assembly": {
                "name": self.assembly_info["assemblyName"],
                "subAssemblyInstances":[],
            },
            "assemblyId": "assembly id stubbed",
            "name": "assembly name stubbed",
            "companyId": "StubbedModelId"
        }
        models['assembly']['subAssemblyInstances'] = self.gen_subassemblyInstances()
        return models

class ComponentAnalyzerMachine(PumpMachine):
    def __init__(self,*args,subtype="component_analyzer",**kwargs):
        super().__init__(*args,**kwargs)
        self.assembly_info=get_assembly_info_map(subtype)
        for sensor in self.sensors:
            sensor.update_names(self.assembly_info["name"],self.assembly_info["display_name"],self.assembly_info["collectors"])
    
    def gen_models(self):
        models ={
            "assembly": {
                "name": self.assembly_info["assemblyName"],
                "subAssemblyInstances":[],
            },
            "assemblyId": "assembly id stubbed",
            "name": "assembly name stubbed",
            "companyId": "StubbedModelId"
        }
        models['assembly']['subAssemblyInstances'] = self.gen_subassemblyInstances()
        return models
    
    
